//+------------------------------------------------------------------+
//|                     Kaufman Adaptive Moving Average with signals |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Gray
#property indicator_color2 Yellow
#property indicator_color3 Magenta
#property indicator_width2 2
#property indicator_width3 2

//
//
//
//
//

extern int    AMAPeriod       = 10;
extern int    AMAPrice        = PRICE_CLOSE;
extern int    Nfast           = 2;
extern int    Nslow           = 30;
extern double GCoeff          = 2.0;
extern double dK              = 2.0; 
extern string _               = "Alerts settings";
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double kAMAbuffer[];
double kAMAupsig[];
double kAMAdownsig[];
double diff[];

//
//
//
//
//

double fastend;
double slowend;
string priceName;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,kAMAbuffer);
   SetIndexBuffer(1,kAMAupsig);   SetIndexStyle(1,DRAW_ARROW); SetIndexArrow(1,159);
   SetIndexBuffer(2,kAMAdownsig); SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,159);
   SetIndexBuffer(3,diff);
   
   
   //
   //
   //
   //
   //
   
      fastend   = (2.0 /(Nfast + 1));
      slowend   = (2.0 /(Nslow + 1));
      switch(AMAPrice)
      {
         case PRICE_CLOSE:    priceName = "Close";    break;  // 0
         case PRICE_OPEN:     priceName = "Open";     break;  // 1
         case PRICE_HIGH:     priceName = "High";     break;  // 2
         case PRICE_LOW:      priceName = "Low";      break;  // 3
         case PRICE_MEDIAN:   priceName = "Median";   break;  // 4
         case PRICE_TYPICAL:  priceName = "Typical";  break;  // 5
         case PRICE_WEIGHTED: priceName = "Weighted"; break;  // 6
      }      
   
   //
   //
   //
   //
   //
   
   IndicatorShortName("Kaufman AMA signals ("+AMAPeriod+")");
   return(0);
}
int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);

           //
           //
           //
           //
           //
                      
           if (Digits==3 || Digits==5)
              double _point=Point*10;
           else      _point=Point;         


   //
   //
   //
   //
   //
   
   for(int i=limit; i>=0; i--)
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,AMAPrice,i);

         if (i > Bars-AMAPeriod) { kAMAbuffer[i] = price; continue; }
   
         //
         //
         //
         //
         //
   
         double efratio = 1.00;
         double noise   = 0.00;
         double smooth;
         double signal;
             signal  = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,AMAPrice,i+AMAPeriod));
             diff[i] = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,AMAPrice,i+1));
                for (int k=0;k<AMAPeriod;k++)
                        noise += diff[i+k];

          //
          //
          //
          //
          //

          if (noise != 0) efratio = signal/noise;
                          smooth        = MathPow(efratio*(fastend-slowend)+slowend,GCoeff);
                          kAMAbuffer[i] = kAMAbuffer[i+1] + smooth*(price-kAMAbuffer[i+1]);

          //
          //
          //
          //
          //
                                    
         kAMAupsig[i]   = EMPTY_VALUE;
         kAMAdownsig[i] = EMPTY_VALUE;
            double ddK=kAMAbuffer[i] - kAMAbuffer[i+1];
               if ((MathAbs(ddK)) > (dK*_point) && (ddK > 0)) kAMAupsig[i]   = kAMAbuffer[i];
               if ((MathAbs(ddK)) > (dK*_point) && (ddK < 0)) kAMAdownsig[i] = kAMAbuffer[i];
                             
   }

   //
   //
   //
   //
   //
      
   if (alertsOn)
   {
      static string doWhat1=""; static datetime doWhen1=0;
      double pPrice  = iMA(NULL,0,1,0,MODE_SMA,AMAPrice,2);
      if (alertsOnCurrent)
            int forTrendBar = 0;
      else      forTrendBar = 1;
      
      //
      //
      //
      //
      //

      if (kAMAupsig[forTrendBar]   != EMPTY_VALUE) doAlert(doWhat1,doWhen1,"signal for BUY");
      if (kAMAdownsig[forTrendBar] != EMPTY_VALUE) doAlert(doWhat1,doWhen1,"signal for SELL");
   }      
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void doAlert(string& previousAlert, datetime& previousTime, string doWhat)
{
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," AMA ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"AMA "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}